package cn.iocoder.yudao.module.bpm.controller.admin.oa;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.replacementCard.BpmOAReplacementCardCreateReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.replacementCard.BpmOAReplacementCardRespVO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.BpmOAReplacementCardDO;
import cn.iocoder.yudao.module.bpm.service.oa.BpmOAReplacementCardService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils.getLoginUserId;

/**
 * OA 补卡申请 Controller
 *
 * @author 符溶馨

 */
@Tag(name = "管理后台 - OA 补卡申请")
@RestController
@RequestMapping("/bpm/oa/replacementCard")
@Validated
public class BpmOAReplacementCardController {

    @Resource
    private BpmOAReplacementCardService replacementCardService;

    @PostMapping("/create")
    @Operation(summary = "创建请求申请")
    public CommonResult<Long> createReplacementCard(@Valid @RequestBody BpmOAReplacementCardCreateReqVO createReqVO) {

        return success(replacementCardService.createReplacementCard(getLoginUserId(), createReqVO));
    }

    @GetMapping("/get")
    @Operation(summary = "获得补卡申请")
    @Parameter(name = "id", description = "编号", required = true, example = "1024")
    public CommonResult<BpmOAReplacementCardRespVO> getReplacementCard(@RequestParam("id") Long id) {

        BpmOAReplacementCardDO replacementCard = replacementCardService.getReplacementCard(id);
        return success(BeanUtils.toBean(replacementCard, BpmOAReplacementCardRespVO.class));
    }

    @GetMapping("/getByProcessInstanceId")
    @Operation(summary = "获得补卡申请")
    @Parameter(name = "processInstanceId", description = "流程实例编号", required = true, example = "1024")
    public CommonResult<BpmOAReplacementCardRespVO> getByProcessInstanceId(@RequestParam("processInstanceId") String processInstanceId) {

        BpmOAReplacementCardDO replacementCard = replacementCardService.getByProcessInstanceId(processInstanceId);
        return success(BeanUtils.toBean(replacementCard, BpmOAReplacementCardRespVO.class));
    }
}
