package cn.iocoder.yudao.module.bpm.service.oa;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.workorderassignrule.BpmOAWorkOrderAssignRulePageReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.workorderassignrule.BpmOAWorkOrderAssignRuleRespVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.workorderassignrule.BpmOAWorkOrderAssignRuleSaveReqVO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.BpmOAWorkOrderAssignRuleDO;

import javax.validation.Valid;
import java.util.Collection;
import java.util.List;

/**
 * BPM OA 工单分配规则 Service 接口
 *
 * @author 系统
 */
public interface BpmOAWorkOrderAssignRuleService {

    /**
     * 创建工单分配规则
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createWorkOrderAssignRule(@Valid BpmOAWorkOrderAssignRuleSaveReqVO createReqVO);

    /**
     * 更新工单分配规则
     *
     * @param updateReqVO 更新信息
     */
    void updateWorkOrderAssignRule(@Valid BpmOAWorkOrderAssignRuleSaveReqVO updateReqVO);

    /**
     * 删除工单分配规则
     *
     * @param id 编号
     */
    void deleteWorkOrderAssignRule(Long id);

    /**
     * 获得工单分配规则
     *
     * @param id 编号
     * @return 工单分配规则
     */
    BpmOAWorkOrderAssignRuleDO getWorkOrderAssignRule(Long id);

    /**
     * 获得工单分配规则分页
     *
     * @param pageReqVO 分页查询
     * @return 工单分配规则分页
     */
    PageResult<BpmOAWorkOrderAssignRuleRespVO> getWorkOrderAssignRulePage(BpmOAWorkOrderAssignRulePageReqVO pageReqVO);

    /**
     * 获得工单分配规则列表（用于导出）
     *
     * @param exportReqVO 查询条件
     * @return 工单分配规则列表
     */
    List<BpmOAWorkOrderAssignRuleRespVO> getWorkOrderAssignRuleList(BpmOAWorkOrderAssignRulePageReqVO exportReqVO);

    /**
     * 根据工单类型获取有效的分配规则
     *
     * @param workOrderType 工单类型
     * @return 分配规则列表
     */
    List<BpmOAWorkOrderAssignRuleDO> getEnabledRulesByType(String workOrderType);

    /**
     * 批量更新规则状态
     *
     * @param ids 规则ID列表
     * @param status 状态（1-启用，0-禁用）
     */
    void updateRuleStatus(Collection<Long> ids, Integer status);

}
