package cn.iocoder.yudao.module.infra.config;

import lombok.Data;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Configuration;

/**
 * MinIO 配置属性类
 *
 * @author AI Assistant
 */
@Data
@Configuration
@ConfigurationProperties(prefix = "minio")
public class MinioConfigProperties {

    /**
     * MinIO 服务地址
     */
    private String endpoint;

    /**
     * 访问密钥
     */
    private String accessKey;

    /**
     * 密钥
     */
    private String secretKey;

    /**
     * 默认存储桶名称
     */
    private String bucketName;

    /**
     * 分片上传性能配置
     */
    private MultipartConfig multipart = new MultipartConfig();

    @Data
    public static class MultipartConfig {
        /**
         * 线程池核心线程数（默认：CPU核心数*2，最小8）
         */
        private int corePoolSize = Math.max(8, Runtime.getRuntime().availableProcessors() * 2);

        /**
         * 线程池最大线程数（默认：核心线程数*4，最大32）
         */
        private int maxPoolSize = Math.min(32, corePoolSize * 4);

        /**
         * 分批合并的批次大小（默认50）
         * 当分片数量超过此值时，将采用分批并行合并策略
         */
        private int batchSize = 50;

        /**
         * 是否启用动态批次大小优化（默认true）
         * 启用后，大文件会使用更大的批次大小以提高合并效率
         */
        private boolean enableDynamicBatchSize = true;

        /**
         * 动态批次大小的最大值（默认100）
         * 当启用动态批次大小时，批次大小不会超过此值
         */
        private int maxDynamicBatchSize = 100;

        /**
         * 是否启用同步清理模式（默认false）
         * 启用后，分片文件清理将同步执行，便于调试验证清理效果
         * 生产环境建议设置为false以提高性能
         */
        private boolean enableSyncCleanup = false;
    }

} 