package cn.iocoder.yudao.module.system.controller.admin.workovertime;

import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Operation;

import javax.validation.constraints.*;
import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;

import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

import cn.iocoder.yudao.framework.excel.core.util.ExcelUtils;

import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;

import cn.iocoder.yudao.module.system.controller.admin.workovertime.vo.*;
import cn.iocoder.yudao.module.system.dal.dataobject.workovertime.WorkOvertimeRuleDO;
import cn.iocoder.yudao.module.system.service.workovertime.WorkOvertimeRuleService;

@Tag(name = "管理后台 - 加班规则")
@RestController
@RequestMapping("/system/work-overtime-rule")
@Validated
public class WorkOvertimeRuleController {

    @Resource
    private WorkOvertimeRuleService workOvertimeRuleService;

    @PostMapping("/createOrUpdate")
    @Operation(summary = "创建或修改加班规则")
    public CommonResult createOrUpdateWorkOvertimeRule(@Valid @RequestBody WorkOvertimeRuleSaveReqVO createReqVO) {
        workOvertimeRuleService.createOrUpdateWorkOvertimeRule(createReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @Operation(summary = "删除加班规则")
    @Parameter(name = "id", description = "编号", required = true)
    public CommonResult<Boolean> deleteWorkOvertimeRule(@RequestParam("id") Long id) {
        workOvertimeRuleService.deleteWorkOvertimeRule(id);
        return success(true);
    }

    @GetMapping("/get")
    @Operation(summary = "获得加班规则")
    @Parameter(name = "id", description = "编号", required = true, example = "1024")
    public CommonResult<WorkOvertimeRuleRespVO> getWorkOvertimeRule(@RequestParam("id") Long id) {
        WorkOvertimeRuleDO workOvertimeRule = workOvertimeRuleService.getWorkOvertimeRule(id);
        return success(BeanUtils.toBean(workOvertimeRule, WorkOvertimeRuleRespVO.class));
    }

    @GetMapping("/page")
    @Operation(summary = "获得加班规则分页")
    public CommonResult<PageResult<WorkOvertimeRuleDO>> getWorkOvertimeRulePage(@Valid WorkOvertimeRulePageReqVO pageReqVO) {
        PageResult<WorkOvertimeRuleDO> pageResult = workOvertimeRuleService.getWorkOvertimeRulePage(pageReqVO);
        return success(pageResult);
    }

    @GetMapping("/export-excel")
    @Operation(summary = "导出加班规则 Excel")
    @OperateLog(type = EXPORT)
    public void exportWorkOvertimeRuleExcel(@Valid WorkOvertimeRulePageReqVO pageReqVO,
              HttpServletResponse response) throws IOException {
        pageReqVO.setPageSize(PageParam.PAGE_SIZE_NONE);
        List<WorkOvertimeRuleDO> list = workOvertimeRuleService.getWorkOvertimeRulePage(pageReqVO).getList();
        // 导出 Excel
        ExcelUtils.write(response, "加班规则.xls", "数据", WorkOvertimeRuleRespVO.class,
                        BeanUtils.toBean(list, WorkOvertimeRuleRespVO.class));
    }

}
