package cn.iocoder.yudao.module.system.service.holiday.holidayworkingage;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.system.controller.admin.holiday.holidayworkingage.vo.HolidayWorkingAgePageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.holiday.holidayworkingage.vo.HolidayWorkingAgeSaveReqVO;
import cn.iocoder.yudao.module.system.dal.dataobject.holiday.holidayworkingage.HolidayWorkingAgeDO;

import javax.validation.Valid;
import java.util.List;

/**
 * 假期额度工龄配置 Service 接口
 *
 * @author 艾楷
 */
public interface HolidayWorkingAgeService {

    /**
     * 创建假期额度工龄配置
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createHolidayWorkingAge(@Valid HolidayWorkingAgeSaveReqVO createReqVO);

    /**
     * 更新假期额度工龄配置
     *
     * @param updateReqVO 更新信息
     */
    void updateHolidayWorkingAge(@Valid HolidayWorkingAgeSaveReqVO updateReqVO);

    /**
     * 删除假期额度工龄配置
     *
     * @param id 编号
     */
    void deleteHolidayWorkingAge(Long id);

    /**
     * 获得假期额度工龄配置
     *
     * @param id 编号
     * @return 假期额度工龄配置
     */
    HolidayWorkingAgeDO getHolidayWorkingAge(Long id);

    /**
     * 获得假期额度工龄配置分页
     *
     * @param pageReqVO 分页查询
     * @return 假期额度工龄配置分页
     */
    PageResult<HolidayWorkingAgeDO> getHolidayWorkingAgePage(HolidayWorkingAgePageReqVO pageReqVO);

    /**
     * 根据假期设置id获取列表
     *
     * @param settingId
     * @return
     */
    List<HolidayWorkingAgeDO> selectBySettingId(Long settingId);

    /**
     * 根据假期设置ids获取列表
     * @param holidaySettingIds
     * @return
     */
    List<HolidayWorkingAgeDO> selectBySettingIds(List<Long> holidaySettingIds);
}
