package cn.iocoder.yudao.module.system.service.rental;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.system.controller.admin.rental.vo.order.RentalOrderPageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.rental.vo.order.RentalOrderRespVO;
import cn.iocoder.yudao.module.system.controller.admin.rental.vo.order.RentalOrderSaveReqVO;
import cn.iocoder.yudao.module.system.dal.dataobject.rental.RentalOrderDO;

import javax.validation.Valid;
import java.math.BigDecimal;

/**
 * 租赁订单 Service 接口
 *
 * @author 符溶馨
 */
public interface RentalOrderService {

    /**
     * 创建租赁订单
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createRentalOrder(@Valid RentalOrderSaveReqVO createReqVO);

    /**
     * 更新租赁订单
     *
     * @param updateReqVO 更新信息
     */
    BigDecimal updateRentalOrder(@Valid RentalOrderSaveReqVO updateReqVO);

    /**
     * 更新租赁订单
     * @param updateReqVO 更新信息
     */
    void update(RentalOrderDO updateReqVO);

    /**
     * 删除租赁订单
     *
     * @param id 编号
     */
    void deleteRentalOrder(Long id);

    /**
     * 获得租赁订单
     *
     * @param id 编号
     * @return 租赁订单
     */
    RentalOrderDO getRentalOrder(Long id);

    /**
     * 获得租赁订单
     *
     * @param orderNo 订单编号
     * @return 租赁订单
     */
    RentalOrderDO getRentalOrder(String orderNo);

    /**
     * 获得租赁订单分页
     *
     * @param pageReqVO 分页查询
     * @return 租赁订单分页
     */
    PageResult<RentalOrderRespVO> getRentalOrderPage(RentalOrderPageReqVO pageReqVO);

    /**
     * 获取订单的剩余押金金额（押金金额-扣款金额）
     * @param orderNo 订单编号
     * @return 剩余押金金额
     */
    BigDecimal getOrderAmount(String orderNo);

    /**
     * 根据订单号获取客户id
     * @param orderNo 订单号
     * @return 客户id
     */
    Long getCustomerIdByOrderNo(String orderNo);
}