package cn.iocoder.yudao.module.system.service.workovertime;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.system.controller.admin.workovertime.vo.WorkOvertimeDeductRulePageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.workovertime.vo.WorkOvertimeDeductRuleSaveReqVO;
import cn.iocoder.yudao.module.system.dal.dataobject.workovertime.WorkOvertimeDeductRuleDO;

import javax.validation.Valid;
import java.util.List;

/**
 * 加班扣除规则 Service 接口
 *
 * @author 艾楷
 */
public interface WorkOvertimeDeductRuleService {

    /**
     * 创建加班扣除规则
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createWorkOvertimeDeductRule(@Valid WorkOvertimeDeductRuleSaveReqVO createReqVO);

    /**
     * 更新加班扣除规则
     *
     * @param updateReqVO 更新信息
     */
    void updateWorkOvertimeDeductRule(@Valid WorkOvertimeDeductRuleSaveReqVO updateReqVO);

    /**
     * 删除加班扣除规则
     *
     * @param id 编号
     */
    void deleteWorkOvertimeDeductRule(Long id);

    /**
     * 获得加班扣除规则
     *
     * @param id 编号
     * @return 加班扣除规则
     */
    WorkOvertimeDeductRuleDO getWorkOvertimeDeductRule(Long id);

    /**
     * 获得加班扣除规则分页
     *
     * @param pageReqVO 分页查询
     * @return 加班扣除规则分页
     */
    PageResult<WorkOvertimeDeductRuleDO> getWorkOvertimeDeductRulePage(WorkOvertimeDeductRulePageReqVO pageReqVO);

    /**
     * 根据规则id获取加班扣除规则列表
     *
     * @param ruleId
     * @return
     */
    List<WorkOvertimeDeductRuleDO> getListByRuleId(Long ruleId);

    /**
     * 批量新增修改删除
     *
     * @param deductRuleLists
     */
    void insertUpdateDeleteBatch(List<List<WorkOvertimeDeductRuleDO>> deductRuleLists);
}
